##THIS IS THE CODE USED IN THE STUDY "Estimating snakebite incidence from mathematical models: A test in Costa Rica"

#USED PACKAGES
install.packages("raster")
install.packages("rgdal")
install.packages("maptools")
install.packages("readxl")
install.packages("rlang")
install.packages("mcr")
install.packages("ggplot2")
install.packages("rJava")
install.packages("rgr")
require(rgr)
require(mcr)
require(ggplot2)
library(raster)
library(rgdal)
library(maptools)
library(readxl)
library(rlang)
library(virtualspecies)
library(dismo)
library(maps)
library(mapdata)
library(vcd)
library(boot)
library(rgeos)
library(sp)
if (Sys.getenv("JAVA_HOME")!="")
  Sys.setenv(JAVA_HOME="")
library(rJava)

data(wrld_simpl)

##IMPORTANT: SET AS WORKING SPACE THE FOLDER WHERE THIS SCRIPT IS LOCATED
  
setwd("C:/BackUp-CB/PHD/1 capitulo/PLOS NTDS/PLOS NTDS/Code")

##ECOLOGICAL NICHE MODELING

##ENVIRONMENTAL DATA
Alt=raster::getData('worldclim', var='alt', res=0.5, lat=10, lon=-75)
names(Alt)=c("Alt")


Bio=raster::getData('worldclim', var='bio', res=0.5, lat=10, lon=-75)
names(Bio)=c("B1","B2","B3","B4","B5","B6","B7","B8","B9","B10","B11","B12","B13","B14","B15","B16","B17","B18","B19")

##CROPPING ENVIRONMENTAL DATASET
CR <- readShapeSpatial("CRI_adm0")
ext <- extent(-86.4, -82, 8, 11.5)
CRcrop = crop(CR,ext)
Climatelist=list(Bio)
Climate=stack(Climatelist)
Climatecrop=crop(Climate,ext)
AltCR=crop(Alt,ext)
rasters.selected=mask(Climatecrop,CRcrop)

##FUNCTION TO CONVERT DMS TO DECIMAL
dms2dec <- function(dms, separators = c("", "", "\'", "\"")) {
  # version 1.0 (25 Sep 3013)
  # dms: a vector (or column) of latitude or longitude in degrees-minutes-seconds-hemisfere,
  # e.g. 41 34' 10.956" N (with or without spaces)
  # separators: the characters that are separating degrees, minutes and seconds in dms
  
  dms <- as.character(dms)
  dms <- gsub(pattern = " ", replacement = "", x = dms)
  for (s in separators) dms <- gsub(pattern = s, replacement = "_splitHere_", x = dms)
  
  splits <- strsplit(dms, split = "_splitHere_")
  n <- length(dms)
  deg <- min <- sec <- hem <- vector("character", n)
  
  for (i in 1:n) {
    deg[i] <- splits[[i]][1]
    min[i] <- splits[[i]][2]
    sec[i] <- splits[[i]][3]
    hem[i] <- splits[[i]][4]
  }
  
  dec <- as.numeric(deg) + (as.numeric(min) / 60) + (as.numeric(sec) / 3600)
  sign <- ifelse (hem %in% c("N", "E"), 1, -1)
  dec <- sign * dec
  return(dec)
} 

#PRESENCE DATA

Snakepoints <- read.csv('baspercr.csv',header=TRUE, sep=";", dec=".") 
Snakepoints$latitude.decimal=dms2dec(Snakepoints$Latitude)
Snakepoints$longitude.decimal=dms2dec(Snakepoints$Longitude)
Points=data.frame(Snakepoints$longitude.decimal, Snakepoints$latitude.decimal)
colnames(Points) = c("lon","lat")


##REMOVING DUPLICATED DATA
dups <- duplicated(Points[, c('lon', 'lat')])
Snakepointscured <- Points[!dups, ]
SN=Snakepointscured

#REMOVING DATA OUTSITE COSTA RICA
coordinates(SN) <- ~lon+lat
crs(SN) <- crs(wrld_simpl)
ovr <- over(SN,wrld_simpl)
cntr <- ovr$NAME
j <- which(cntr=="Costa Rica")
SNR=Snakepointscured[j,]
SREAL=SNR

##FILTERING BY ALTITUDE 
altitudecond=extract(AltCR, SREAL)
b=which(!is.na(altitudecond)&altitudecond<1400)
SREAL=SREAL[b,]


##SELECTING TRAINING AND TESTING DATA
fold <- kfold(SREAL, k=5)
pres_test <- SREAL[fold == 1, ]
pres_train <- SREAL[fold != 1, ]
##EXTRACTING ENVIRONMENTAL DATA
v <- extract(rasters.selected, pres_train)
#REMOVING NA
i <- which(apply(is.na(v), 1,  sum) > 0)
pres_train <- pres_train[-i, ]

##CREATING MAXENT DATASETS
pvtest=data.frame(extract(rasters.selected,pres_test)) 
bg=randomPoints(rasters.selected,1000)
avtest=data.frame(extract(rasters.selected,bg))


#MAXENT (IMPORTANT: YOU MUST DOWNLOAD THE .jar FILE OR UNZIP THE FILE "maxent" INTO YOUR DISMO FOLDER ON R PACKAGES)
MESnakes <- maxent(x=rasters.selected, p=pres_train, a=bg,args=c('randomtestpoints=20','responsecurves=true','jackknife=true'))

#PERFORMANCE TESTS
e1=evaluate(MESnakes,p=pvtest,a=avtest)
e1
testp=predict(MESnakes,pvtest)
head(testp)
testa=predict(MESnakes,avtest)
e2=evaluate(p=testp,a=testa)
threshold(e2)

##PREDICTING DISTRIBUTION
r <- predict(MESnakes, rasters.selected, progress='text', ext=ext,args=c("outputformat=logistic"))
writeRaster(r, filename="NICHO1.tif",format="GTiff",overwrite=TRUE)
##LOADING SELECTED NICHE MODEL
r=raster("NICHO.tif")
##REMOVING SCORES BELOW THE SELECTED TRESHOLD
Nicho=Which(r>0.207)
#REMOVING AREAS OVER 1200 M.A.S.L.
Nicho=Nicho*Which(AltCR<1200)


#INCIDENCE DATASETS
Incidencia <- read_excel("Incidence.xlsx",sheet = "Sheet6")
DistrictInfo <-read_excel("Incidence.xlsx",sheet = "Sheet5")

##FILTERING DATASETS
colnames=c("CODDIST","NPROV","NCANTON","NDIST","AO","CASOS")
Incidata=data.frame(matrix(ncol=length(colnames),nrow=0))
colnames(Incidata)=colnames
##TOTAL NUMBER OF CASES BY YEAR
for (i in unique(DistrictInfo$`Posible lugar de mordedura`)){
  for (j in unique(DistrictInfo$ano)){
    a=DistrictInfo[(DistrictInfo$`Posible lugar de mordedura`==i & DistrictInfo$ano == j),]
    b=na.omit(Incidencia[(Incidencia$`Codigo Distrito`==i),])
    c=data.frame(i,b$PROVINCIA[1],b$NCANTON[1],b$NDISTRITO[1],j,length(a$`Distrto codigo`))
    colnames(c)=colnames
    Incidata=rbind(Incidata,c)
  }
  
}



#ADDING INFORMATION OF ENCOUNTER FREQUENCY TO NICHE PREDICTION
Zonas=raster("Zonas.tif")
Zonasfin=projectRaster(Zonas,Nicho,method="ngb")
step=400

lowpacsur=Which(AltCR < step & Zonasfin==3,cell=TRUE)
midpacsur=Which(AltCR >= step & AltCR <2*step & Zonasfin==3,cell=TRUE) 
highpacsur=Which(AltCR >= 2*step & Zonasfin==3,cell=TRUE) 
lowpacmid=Which(AltCR < step & Zonasfin==1,cell=TRUE) 
midpacmid=Which(AltCR >= step & AltCR <2*step & Zonasfin==1,cell=TRUE) 
highpacmid=Which(AltCR >= 2*step & Zonasfin==1,cell=TRUE) 
lowcar=Which(AltCR < step & Zonasfin==2,cell=TRUE) 
midcar=Which(AltCR >= step & AltCR <2*step & Zonasfin==2,cell=TRUE) 
highcar=Which(AltCR >= 2*step & Zonasfin==2,cell=TRUE) 



Average=Nicho
Average[lowpacsur]=0.33871
Average[midpacsur]=0.145863
Average[highpacsur]=0.0333333
Average[lowpacmid]=0.33871
Average[midpacmid]=0.145863
Average[highpacmid]=0.0333333
Average[lowcar]=0.1245
Average[midcar]=0.132342
Average[highcar]=0.0333333
Average=Average*Nicho

writeRaster(Average,filename = "F.tif",format="GTiff", overwrite=TRUE)
##DISRICT DATA

Distritos=readOGR("DistritosFINALFFF.shx")

##EXTRACTING AVERAGE ENCOUNTER FREQUENCY PER EACH DISTRICT

ex = extract(Average, Distritos, fun=mean, na.rm=TRUE, df=TRUE)
ex1=extract(r,Distritos,fun=mean,na.rm=TRUE, df=TRUE)


##CREATING THE MODEL DATASET
ex$NombreDistrito=Distritos$NDISTRITO
ex$AreaDistrito=Distritos$AREA
ex$CodDIST=Distritos$CODDIST
ex$nicho=ex1$NICHO
##AVERAGE RURAL POPULATION
Poblacion=0.01*Distritos$PorcRur200*(0.5*(as.numeric(levels(Distritos$POB_1997_M))[Distritos$POB_1997_M]+as.numeric(levels(Distritos$POB_1997_H))[Distritos$POB_1997_H]+as.numeric(levels(Distritos$POB_2000_M))[Distritos$POB_2000_M]+as.numeric(levels(Distritos$POB_2000_H))[Distritos$POB_2000_H]))
ex$Poblacion=Poblacion
ex$NombreCanton=Distritos$NCANTON
ex$NombreProvincia=Distritos$PROVINCIA
ex=ex[ex$Poblacion>0,]
ex=ex[is.na(ex$Poblacion)==FALSE,]
FinalDatabase=ex[ex$NombreCanton==1,]

##Filtering the dataset
for (i in unique(ex$NombreCanton)){
  for (j in unique(ex$NombreDistrito)){
    a=subset(ex,NombreCanton==i)
    b=subset(a,NombreDistrito==j)
    if (nrow(b)>1){
      a=0
      for (k in unique(b$AreaDistrito)){
        c=subset(b,AreaDistrito==k)
        
        if(a<unique(c$AreaDistrito)){
          d=c
          a=c$AreaDistrito
        }
        
      }
      FinalDatabase=rbind(FinalDatabase,d)
    } else {
      FinalDatabase=rbind(FinalDatabase,b)
    }
    
  }
  
}


colnamesFIN=c("CODDIST","NPROV","NCANTON","NDIST","AO","CASOS","MeanFreqEnc","Poblacion","ALT","NICHO")
FDB=data.frame(matrix(ncol=length(colnamesFIN),nrow=0))
colnames(FDB)=colnamesFIN
Incidatafin=na.omit(Incidata)
##CREATING FINAL DATASET
for (i in unique(Incidatafin$CODDIST)){
  a=subset(FinalDatabase,CodDIST==i)
  b=subset(Incidatafin,CODDIST==i)
  b$MeanFreqEnc=a$layer
  b$Poblacion=a$Poblacion
  b$ALT=a$ALT
  b$NICHO=a$nicho
  FDB=rbind(FDB,b)
}
##COMPUTING OUR ESTIMATION
FDB$EstimAnual=(FDB$MeanFreqEnc*FDB$Poblacion)
#FILTERING DISTRICTS WITH B. asper
cond=FDB$MeanFreqEnc>0

FFDB=subset(FDB,cond)
##COMPUTING NATIONAL INCIDENCE PER YEAR
NacIc=aggregate(FFDB$CASOS, by=list(Year=FFDB$AO), FUN=sum)


##COMPUTING AVERAGE INCIDENCE FOR THE STUDY YEARS

colnamesfff=c("CODDIST","NPROV","NCANTON","NDIST","CASES","MEANFREQ","POB","ALT","ESTIM","ESTIMNICHO")
FFFDB=data.frame(matrix(ncol=length(colnamesfff),nrow=0))
colnames(FFFDB)=colnamesfff


for (i in unique(FFDB$CODDIST)){
  a=subset(FFDB,CODDIST==i)
  meaninc=mean(a$CASOS)
  b=data.frame(i,a$NPROV[1],a$NCANTON[1],a$NDIST[1],meaninc,mean(a$MeanFreqEnc),mean(a$Poblacion),mean(a$ALT),mean(a$EstimAnual),mean(a$Estimanualnicho))
  colnames(b)=colnamesfff
  FFFDB=rbind(FFFDB,b)
}

#Shapefile with incidence, encounter frequency, rural population and model estimation
ex = extract(Average, Distritos, fun=mean, na.rm=TRUE, df=TRUE)
Distritos@data$Average=ex$layer
Distritos@data$PRURAL=Poblacion
Distritos@data$Incidence=0
for (i in unique(Distritos@data$CODDIST)){
  if (!is_empty(FFFDB$CASES[FFFDB$CODDIST==i])){
    Distritos@data$Incidence[Distritos@data$CODDIST==i]=FFFDB$CASES[FFFDB$CODDIST==i]
  }}
Distritos$Estimation=0.4711+1.8e-6*(Distritos@data$PRURAL*Distritos@data$Average*48*4*10)
Distritos$Estimation[Distritos$PRURAL<0]=0
writeOGR(Distritos, ".", "FinalDB", driver="ESRI Shapefile")
##Encounter frequency in encounters per year (8 work hours*5 work days in a week*4 weeks in a month*10 work months)
FFFDB$MEANFREQ=FFFDB$MEANFREQ*48*4*10
FFFDB$ESTIM=FFFDB$ESTIM*48*4*10
##PERFORMING LINEAR REGRESSIONS
Y=FFFDB$CASES
X=FFFDB$ESTIM
dist=FFFDB$NDIST
canton=FFFDB$NCANTON
mod=lm(Y~X)
mod1=lm(Y~X+0)
mod2=lm(Y~FFFDB$MEANFREQ)
mod3=lm(Y~FFFDB$POB)
mod4=lm(Y~FFFDB$POB+FFFDB$MEANFREQ)
#COMPUTING R SQUARED
##TOTAL SUM OF SQUARES
SST=var(Y)*(length(Y)-1)
##REGRESSION SUM OF SQUARES
SSE=sum(mod$residuals^2)
SSE1=sum(mod1$residuals^2)
SSE2=sum(mod2$residuals^2)
SSE3=sum(mod3$residuals^2)
SSE4=sum(mod4$residuals^2)
##R squared
R=1-(SSE/SST)
R1=1-(SSE1/SST)
R2=1-(SSE2/SST)
R3=1-(SSE3/SST)
R4=1-(SSE4/SST)
##AIC
AIC=AIC(mod)
AIC1=AIC(mod1)
AIC2=AIC(mod2)
AIC3=AIC(mod3)
AIC4=AIC(mod4)
##NATIONAL PREDICTED INCIDENCE
new1=data.frame(X)
P=sum(predict(mod,new1,level=0.99))
P1=sum(predict(mod1,new1,level=0.99))
P2=sum(predict(mod2,new1,level=0.99))
P3=sum(predict(mod3,new1,level=0.99))
P4=sum(predict(mod4,new1,level=0.99))
##NATIONAL PREDICTION INTERVAL
PL=sum(predict(mod,new1,interval='confidence',level=0.99)[,2])
PH=sum(predict(mod,new1,interval='confidence',level=0.99)[,3])
P1L=sum(predict(mod1,new1,interval='confidence',level=0.99)[,2])
P1H=sum(predict(mod1,new1,interval='confidence',level=0.99)[,3])
P2L=sum(predict(mod2,new1,interval='confidence',level=0.99)[,2])
P2H=sum(predict(mod2,new1,interval='confidence',level=0.99)[,3])
P3L=sum(predict(mod3,new1,interval='confidence',level=0.99)[,2])
P3H=sum(predict(mod3,new1,interval='confidence',level=0.99)[,3])
P4L=sum(predict(mod4,new1,interval='confidence',level=0.99)[,2])
P4H=sum(predict(mod4,new1,interval='confidence',level=0.99)[,3])
##PLOTTING REGRESSION
new=data.frame(X=seq(0,6.5e6,length.out=length(X)))
predicted_values=predict(mod,new,interval="confidence",level=0.99)
Prediction=predicted_values[,1]
CI_low=predicted_values[,2]
CI_high=predicted_values[,3]
new1=data.frame(X)
predictnorm=predict(mod,new,interval="prediction",level=0.99)
PI_low=predictnorm[,2]
PI_high=predictnorm[,3]
PI_1=predict(mod,new1,interval='prediction',level=0.99)[,2]
PI_2=predict(mod,new1,interval='prediction',level=0.99)[,3]
Upout=Y-PI_2
Downout=PI_1-Y
mpi = data.frame(dist,canton,new,X,Y,Prediction,CI_low,CI_high,PI_low,PI_high,Upout,Downout)

ggplot(mpi, aes(x = X.1,y=Y))+
  theme(panel.background = element_rect(fill = "white", colour = "black"))+
  coord_cartesian(xlim=c(0,6.1e6),ylim=c(0,16))+
  geom_vline(xintercept=seq(0, 6.1e6, length.out = 5),color="gray")+
  geom_hline(yintercept=seq(0, 20, length.out = 5),color="gray")+
  scale_x_continuous(name="Encounter frequency",breaks=seq(0,6.1e6,1.22e6))+
  geom_line(aes(y=PI_low,x=X), colour="black",linetype="dashed") +
  geom_line(aes(y=PI_high,x=X), colour="black",linetype="dashed") +
  geom_point(shape = 21,fill="white", colour = "black", size = 1.7, stroke = 1.1) +
  geom_text(data=subset(mpi, Upout>0|Downout>0),aes(label=dist,hjust=0, vjust=0),size=2)+
  geom_ribbon(aes(ymin = CI_low, ymax = CI_high, x=X), fill = "red", alpha = 0.2) +
  geom_line(aes(y = Prediction,x=X), colour = "red", size = 1)

##RESIDUALS BEHAVIOR
res=mod$residuals
resi=res
Predictionmod=predict(mod1,new1)
pred=Predictionmod
cor.test(resi,pred)
plot(resi~pred)

